/* dojo/_base/Color */

declare namespace Color {
    type ColorValue = [number, number, number];
    type ColorValueAlpha = [number, number, number, number];

    interface ColorObject {
        r: number;
        g: number;
        b: number;
        a?: number;
    }

    interface ColorNamed {
        'black': ColorValue;
        'silver': ColorValue;
        'gray': ColorValue;
        'white': ColorValue;
        'maroon': ColorValue;
        'red': ColorValue;
        'purple': ColorValue;
        'fuchsia': ColorValue;
        'green': ColorValue;
        'lime': ColorValue;
        'olive': ColorValue;
        'yellow': ColorValue;
        'navy': ColorValue;
        'blue': ColorValue;
        'teal': ColorValue;
        'aqua': ColorValue;
        'transparent': [number, number, number, number];
    }

    interface Color {
        r: number;
        g: number;
        b: number;
        a: number;
        _set(r: number, g: number, b: number, a: number): void;

        /** Takes a named string, hex string, array of rgb or rgba values,
         * an object with r, g, b, and a properties, or another `Color` object
         * and sets this color instance to that value.
         */
        setColor(color: ColorValue | ColorValueAlpha | ColorObject | string): Color;

        /**
         * Ensures the object has correct attributes
         */
        sanitize(): Color;

        /**
         * Returns 3 component array of rgb values
         */
        toRgb(): ColorValue;

        /**
         * Returns a 4 component array of rgba values from the color represented by
         * this object.
         */
        toRgba(): ColorValueAlpha;

        /**
         * Returns a CSS color string in hexadecimal representation
         */
        toHex(): string;

        /**
         * Returns a css color string in rgb(a) representation
         */
        toCss(includeAlpha?: boolean): string;

        /**
         * Returns a visual representation of the color
         */
        toString(): string;
    }

    interface ColorConstructor {
        new(color: ColorValue | ColorValueAlpha | ColorObject | string): Color;
        prototype: Color;

        /**
         * Dictionary list of all CSS named colors, by name. Values are 3-item arrays with corresponding RG and B values.
         */
        named: ColorNamed;

        /**
         * Blend colors end and start with weight from 0 to 1, 0.5 being a 50/50 blend,
         * can reuse a previously allocated Color object for the result
         */
        blendColors(start: Color, end: Color, weight: number, obj?: Color): Color;

        /**
         * Returns a `Color` instance from a string of the form
         * "rgb(...)" or "rgba(...)". Optionally accepts a `Color`
         * object to update with the parsed value and return instead of
         * creating a new object.
         */
        fromRgb(color: string, obj?: Color): Color;

        /**
         * Converts a hex string with a '#' prefix to a color object.
         * Supports 12-bit #rgb shorthand. Optionally accepts a
         * `Color` object to update with the parsed value.
         */
        fromHex(color: string, obj?: Color): Color;

        /**
         * Builds a `Color` from a 3 or 4 element array, mapping each
         * element in sequence to the rgb(a) values of the color.
         */
        fromArray(color: ColorValue | ColorValueAlpha, obj?: Color): Color;

        /**
         * Parses `str` for a color value. Accepts hex, rgb, and rgba
         * style color values.
         */
        fromString(str: string, obj?: Color): Color;
    }

    /* dojo/colors */

    interface ColorNamed {
        'aliceblue': ColorValue;
        'antiquewhite': ColorValue;
        'aquamarine': ColorValue;
        'azure': ColorValue;
        'beige': ColorValue;
        'bisque': ColorValue;
        'blanchedalmond': ColorValue;
        'blueviolet': ColorValue;
        'brown': ColorValue;
        'burlywood': ColorValue;
        'cadetblue': ColorValue;
        'chartreuse': ColorValue;
        'chocolate': ColorValue;
        'coral': ColorValue;
        'cornflowerblue': ColorValue;
        'cornsilk': ColorValue;
        'crimson': ColorValue;
        'cyan': ColorValue;
        'darkblue': ColorValue;
        'darkcyan': ColorValue;
        'darkgoldenrod': ColorValue;
        'darkgray': ColorValue;
        'darkgreen': ColorValue;
        'darkgrey': ColorValue;
        'darkkhaki': ColorValue;
        'darkmagenta': ColorValue;
        'darkolivegreen': ColorValue;
        'darkorange': ColorValue;
        'darkorchid': ColorValue;
        'darkred': ColorValue;
        'darksalmon': ColorValue;
        'darkseagreen': ColorValue;
        'darkslateblue': ColorValue;
        'darkslategray': ColorValue;
        'darkslategrey': ColorValue;
        'darkturquoise': ColorValue;
        'darkviolet': ColorValue;
        'deeppink': ColorValue;
        'deepskyblue': ColorValue;
        'dimgray': ColorValue;
        'dimgrey': ColorValue;
        'dodgerblue': ColorValue;
        'firebrick': ColorValue;
        'floralwhite': ColorValue;
        'forestgreen': ColorValue;
        'gainsboro': ColorValue;
        'ghostwhite': ColorValue;
        'gold': ColorValue;
        'goldenrod': ColorValue;
        'greenyellow': ColorValue;
        'grey': ColorValue;
        'honeydew': ColorValue;
        'hotpink': ColorValue;
        'indianred': ColorValue;
        'indigo': ColorValue;
        'ivory': ColorValue;
        'khaki': ColorValue;
        'lavender': ColorValue;
        'lavenderblush': ColorValue;
        'lawngreen': ColorValue;
        'lemonchiffon': ColorValue;
        'lightblue': ColorValue;
        'lightcoral': ColorValue;
        'lightcyan': ColorValue;
        'lightgoldenrodyellow': ColorValue;
        'lightgray': ColorValue;
        'lightgreen': ColorValue;
        'lightgrey': ColorValue;
        'lightpink': ColorValue;
        'lightsalmon': ColorValue;
        'lightseagreen': ColorValue;
        'lightskyblue': ColorValue;
        'lightslategray': ColorValue;
        'lightslategrey': ColorValue;
        'lightsteelblue': ColorValue;
        'lightyellow': ColorValue;
        'limegreen': ColorValue;
        'linen': ColorValue;
        'magenta': ColorValue;
        'mediumaquamarine': ColorValue;
        'mediumblue': ColorValue;
        'mediumorchid': ColorValue;
        'mediumpurple': ColorValue;
        'mediumseagreen': ColorValue;
        'mediumslateblue': ColorValue;
        'mediumspringgreen': ColorValue;
        'mediumturquoise': ColorValue;
        'mediumvioletred': ColorValue;
        'midnightblue': ColorValue;
        'mintcream': ColorValue;
        'mistyrose': ColorValue;
        'moccasin': ColorValue;
        'navajowhite': ColorValue;
        'oldlace': ColorValue;
        'olivedrab': ColorValue;
        'orange': ColorValue;
        'orangered': ColorValue;
        'orchid': ColorValue;
        'palegoldenrod': ColorValue;
        'palegreen': ColorValue;
        'paleturquoise': ColorValue;
        'palevioletred': ColorValue;
        'papayawhip': ColorValue;
        'peachpuff': ColorValue;
        'peru': ColorValue;
        'pink': ColorValue;
        'plum': ColorValue;
        'powderblue': ColorValue;
        'rosybrown': ColorValue;
        'royalblue': ColorValue;
        'saddlebrown': ColorValue;
        'salmon': ColorValue;
        'sandybrown': ColorValue;
        'seagreen': ColorValue;
        'seashell': ColorValue;
        'sienna': ColorValue;
        'skyblue': ColorValue;
        'slateblue': ColorValue;
        'slategray': ColorValue;
        'slategrey': ColorValue;
        'snow': ColorValue;
        'springgreen': ColorValue;
        'steelblue': ColorValue;
        'tan': ColorValue;
        'thistle': ColorValue;
        'tomato': ColorValue;
        'turquoise': ColorValue;
        'violet': ColorValue;
        'wheat': ColorValue;
        'whitesmoke': ColorValue;
        'yellowgreen': ColorValue;
    }

    interface ColorConstructor {
        /**
         * creates a greyscale color with an optional alpha
         */
        makeGrey(g: number, a?: number): Color;
    }

}

type Color = Color.Color;
declare const Color: Color.ColorConstructor;
export = Color;
